//+------------------------------------------------------------------+
//|                                                  StepMA_v7.1.mq4 |
//|                                Copyright  2006, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+

#property copyright "Copyright  2006, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Magenta
#property indicator_color2 Aqua
#property indicator_width1 4
#property indicator_width2 4

extern ENUM_TIMEFRAMES TimeFrame = PERIOD_CURRENT;
extern int     Length      = 21;      // Volty Length
extern double  Kv          = 3;       // Sensivity Factor
extern int     StepSize    = 20;      // Constant Step Size (if need)
extern double  Variance    = 1;
extern double  Mean        = 0.0;
extern int     MA_Mode     = 0;       // Volty MA Mode : 0-SMA, 1-LWMA 
extern int     Advance     = 0;       // Offset
extern bool    HighLow     = false;   // High/Low Mode Switch (more sensitive)
extern int     ColorMode   = 1;       // Color Mode Switch
extern int     BarsNumber  = 1000;

extern bool    ShowLines          = true;
extern color   LinesUpColor       = Aqua;
extern color   LinesDnColor       = Magenta;
extern string  LinesIdentifier    = "StepMa lines";

extern bool    alertsOn            = true;
extern bool    alertsOnCurrent     = true;
extern bool    alertsMessage       = true;
extern bool    alertsNotification  = false;
extern bool    alertsSound         = false;
extern bool    alertsEmail         = false;     

double LineBuffer[];
double UpBuffer[];
double DnBuffer[];
double smin[];
double smax[];
double trend[];
double coeffs[];

bool   returnBars;
string indicatorFileName;

double StepMA=0, ATR0=0,ATRmax=-100000,ATRmin=1000000;
int limit;

//---- StepSize Calculation

   double StepSizeCalc ( int Len, double Km, int Size, int k)
   {

   double result;
   if( Size==0 ) 
   {
        double AvgRange=0;
	     double Weight = 0;
	     for (int i=Len-1;i>=0;i--)
	     { 
            if(MA_Mode==0) double alfa= 1.0; else alfa= 1.0*(Len-i)/Len; 
            AvgRange+= alfa*(High[k+i]-Low[k+i]);
            Weight += alfa;
        }
	     ATR0 = AvgRange/Weight;
	
	if (ATR0>ATRmax) ATRmax=ATR0;
	if (ATR0<ATRmin) ATRmin=ATR0;
	
	result=MathRound(0.5*Km*(ATRmax+ATRmin)/Point); 
	}
	else
	result=Km*StepSize;
   
   return(result);
   }

//---- StepMA Calculation   
   
   double StepMACalc (int Length, bool HL, double Size, int k)
   {
   int counted_bars=IndicatorCounted();
   double result;
   
   if (HL)
	  {
	  smax[k]=iPdfma(Low[k] ,Length,coeffs,k,0)+2.0*Size*Point;
	  smin[k]=iPdfma(High[k],Length,coeffs,k,1)-2.0*Size*Point;
     } 	
	else  	 
	  {
	  smax[k]=iPdfma(Close[k],Length,coeffs,k,0)+2.0*Size*Point;
	  smin[k]=iPdfma(Close[k],Length,coeffs,k,0)-2.0*Size*Point;
	  }
	  if (counted_bars==0){smax[limit+1]=smax[limit];smin[limit+1]=smin[limit];trend[limit+1]=0;}
	  
	  trend[k]=trend[k+1];   
	  
	  if (Close[k]>smax[k+1]) trend[k]=1;   
	 
	  if (Close[k]<smin[k+1]) trend[k]=-1;  
	 
	  if(trend[k]>0)
	  {
	  if(smin[k]<smin[k+1]) smin[k]=smin[k+1];
	  result=smin[k]+Size*Point;
	  } 
	  else
	  {
	  if(smax[k]>smax[k+1]) smax[k]=smax[k+1];
	  result=smax[k]-Size*Point;
	  }

     return(result); 
     } 
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
#define maxx 3.5
  int init()
  {
   string short_name;
//---- indicator line
   IndicatorBuffers(6);
   
   SetIndexStyle(0,DRAW_LINE);
   SetIndexStyle(1,DRAW_LINE);

   SetIndexArrow(1,159);
   SetIndexArrow(2,159);
   
   SetIndexBuffer(0,LineBuffer);
   SetIndexBuffer(1,UpBuffer);
   SetIndexBuffer(2,DnBuffer);
   
   SetIndexShift(0,Advance);
   SetIndexShift(1,Advance);
   SetIndexShift(2,Advance);
   
   SetIndexBuffer(3,smin);
   SetIndexBuffer(4,smax);
   SetIndexBuffer(5,trend);
   
   short_name="StepMA MTF ("+Length+","+Kv+","+StepSize+")";
   IndicatorShortName(short_name);
   
   SetIndexDrawBegin(0,Length);
   SetIndexDrawBegin(1,Length);
   SetIndexDrawBegin(2,Length);

   ArrayResize(coeffs,Length);   
   double step = maxx/(Length-1); for(int i=0; i<Length; i++) coeffs[i] = pdf(i*step,Variance,Mean*maxx);
   
   indicatorFileName = WindowExpertName();
   returnBars        = TimeFrame == -99;
   TimeFrame         = MathMax(TimeFrame,_Period);

   return(0);
   }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {deleteArrows();
   return(0);
  }     
//+------------------------------------------------------------------+
//| StepMA_v7.2                                                        |
//+------------------------------------------------------------------+
int start()
{
   int counted_bars=IndicatorCounted();
   int shift,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit=MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { LineBuffer[0] = MathMin(limit+1,Bars-1); return(0); }
      
   if (TimeFrame != Period())
      {
         limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
         
         for (shift=limit; shift>=0; shift--)
            {
               int y = iBarShift(NULL,TimeFrame,Time[shift]);
                  LineBuffer[shift]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Length,Kv,StepSize,Variance,Mean,MA_Mode,Advance,HighLow,ShowLines,LinesUpColor,LinesDnColor,LinesIdentifier,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,0,y);
                  UpBuffer[shift]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Length,Kv,StepSize,Variance,Mean,MA_Mode,Advance,HighLow,ShowLines,LinesUpColor,LinesDnColor,LinesIdentifier,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,1,y);
                  DnBuffer[shift]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Length,Kv,StepSize,Variance,Mean,MA_Mode,Advance,HighLow,ShowLines,LinesUpColor,LinesDnColor,LinesIdentifier,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,2,y);
                  trend[shift]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Length,Kv,StepSize,Variance,Mean,MA_Mode,Advance,HighLow,ShowLines,LinesUpColor,LinesDnColor,LinesIdentifier,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,5,y);
                  
                  /*if (!Interpolate || y==iBarShift(NULL,TimeFrame,Time[i-1])) continue;
         
                     datetime time = iTime(NULL,TimeFrame,y);
                        for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
                        for(int k = 1; k < n; k++) 
                              {
                                 MacdBuffer[i+k] = MacdBuffer[i] + (MacdBuffer[i+n] - MacdBuffer[i])*k/n;
                                 SignalBuffer[i+k] = SignalBuffer[i] + (SignalBuffer[i+n] - SignalBuffer[i])*k/n;
                              }*/
            }
      return(0);
      }

      
   for(shift=limit;shift>=0;shift--) 
         {	
            int Step = StepSizeCalc( Length, Kv, StepSize, shift);
      				
            StepMA = StepMACalc ( Length, HighLow, Step, shift);
      	
         	if ( trend[shift]>0 )
               {
                  UpBuffer[shift]=StepMA-Step*Point;
                  LineBuffer[shift]= UpBuffer[shift];
                  DnBuffer[shift]=EMPTY_VALUE;
               }
            if ( trend[shift]<0 )
               {
                  DnBuffer[shift]=StepMA+Step*Point;
                  LineBuffer[shift]= DnBuffer[shift];
                  UpBuffer[shift]=EMPTY_VALUE;
               }
              	
      	//---------------------------------------------
         	      	   
         	if (ShowLines)
               {
                  deleteArrow(Time[shift]);
                  if (trend[shift] != trend[shift+1])
                     {
                        if (trend[shift] > 0)  drawArrow(shift,true);
                        if (trend[shift] < 0)  drawArrow(shift,false);
                     }
               }
         }
      
//--------- end for(shift=limit;shift>=0;shift--) 
   	
      	if (alertsOn)
            {
               if (alertsOnCurrent)
                     int whichBar = 0;
               else      whichBar = 1;
               if (trend[whichBar] != trend[whichBar+1])
               if (trend[whichBar] > 0)
                     doAlert(" BUY Signal @ ");
               else  doAlert(" SELL Signal @ ");
             }
                
   	return(0);	
 }

//------------------------------------------------------------

void drawArrow(int i,bool up)
{
   string name = LinesIdentifier+":"+Time[i];
     
         ObjectCreate(name,OBJ_VLINE,0,Time[i],0);
         ObjectSet(name,OBJPROP_STYLE,1);
         ObjectSet(name,OBJPROP_WIDTH,1);
         if (up)
               ObjectSet(name,OBJPROP_COLOR,LinesUpColor);
         else  ObjectSet(name,OBJPROP_COLOR,LinesDnColor);
}

void deleteArrows()
{
   string lookFor       = LinesIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
}

void deleteArrow(datetime time)
{
   string lookFor = LinesIdentifier+":"+time; ObjectDelete(lookFor);
}

//------------------------------------------------------------------

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) 
      {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          message = "StepMA ("+Symbol()+",TF "+Period()+"): "+doWhat+DoubleToStr(Close[0],Digits);
             if (alertsMessage)      Alert(message);
             if (alertsNotification) SendNotification(message);
             if (alertsEmail)        SendMail(StringConcatenate(Symbol()," StepMa pdf "),message);
             if (alertsSound)        PlaySound("alert2.wav");
      }
}

//------------------------------------------------------------------

#define Pi 3.141592653589793238462643

double pdf(double x, double variance=1.0, double mean=0)
      { return((1.0/MathSqrt(2*Pi*MathPow(variance,2))*MathExp(-MathPow(x-mean,2)/(2*MathPow(variance,2))))); }

double workPdfma[][2];
double iPdfma(double price, double period, double& coeffs[], int r, int instanceNo=0)
{
   if (ArraySize(workPdfma)!= Bars) ArrayResize(workPdfma,Bars); r = Bars-r-1;
     
   workPdfma[r][instanceNo] = price;
      double sumw = coeffs[0];
      double sum  = coeffs[0]*price;

      for(int k=1; k<period && (r-k)>=0; k++)
      {
         double weight = coeffs[k];
                sumw  += weight;
                sum   += weight*workPdfma[r-k][instanceNo];  
      }             
      return(sum/sumw);
}

//-------------------------------------------------------------------

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
//-------------------------------------------------------------------